﻿#Requires -Version 3
#Requires -Module ReportingServicesTools

[CmdletBinding(DefaultParameterSetName = 'Default')]
param(
    [Parameter(ParameterSetName = 'Default')]
    [Parameter(ParameterSetName = 'DeployDataSource')]
    [ValidateNotNullorEmpty()]
    [String]$ReportServerURI = 'http://localhost/ReportServer/',

    [Parameter(ParameterSetName = 'Default')]
    [Parameter(ParameterSetName = 'DeployDataSource')]
    [ValidateNotNullorEmpty()]
    [String]$ReportFolderName = "iServiceCRM",
    
    [Parameter(ParameterSetName = 'Default')]
    [Parameter(ParameterSetName = 'DeployDataSource')]
    [ValidateNotNullorEmpty()]
    [String]$DataSourceFolderName = "Data Sources",
      
    [Parameter(ParameterSetName = 'Default')]
    [Parameter(ParameterSetName = 'DeployDataSource')]
    [ValidateNotNullorEmpty()]
    [String]$DataSourceName = "iServiceCRM",

    [Parameter(ParameterSetName = 'DeployDataSource')]
    [Switch]$DeployDataSource,

    [Parameter(ParameterSetName='DeployDataSource',Mandatory)]
    [ValidateNotNullorEmpty()]
    [String]$DataSourceServerName,
      
    [Parameter(ParameterSetName='DeployDataSource',Mandatory)]
    [ValidateNotNullorEmpty()]
    [String]$DataSourceDatabaseName,
      
    [Parameter(ParameterSetName='DeployDataSource',Mandatory)]
    [ValidateNotNullorEmpty()]
    [String]$DataSourceUserName,

    [Parameter(ParameterSetName='DeployDataSource')]
    [String]$DataSourcePassword
     
    )

$ReportsInBuild = Get-ChildItem '.\iServiceCRM' -Filter '*.rdl'
$ImagesInBuild = Get-ChildItem '.\iServiceCRM' -Filter '*.png'
$ReportsNotInBuild = @()

#Obfuscate the data source password entry if it wasn't already provided as an parameter
if ($DeployDataSource) {
    if ($DataSourcePassword) {
        $SecureDSPassword = $DataSourcePassword | ConvertTo-SecureString -AsPlainText -Force
    }
    else {
        $SecureDSPassword = Read-Host -Prompt "Data Source Password:" -AsSecureString
    }
}

Connect-RsReportServer -ReportServerUri $ReportServerURI

if ($DeployDataSource)

    {
        #Prepare Data Source connection details
        $DSConnectionString = "Data Source=$DataSourceServerName;Initial Catalog=$DataSourceDatabaseName"
        $DSCredential = New-Object System.Management.Automation.PSCredential ($DataSourceUserName, $SecureDSPassword)

        #Create the folder for data sources on the report server if it doesn't already exist  
        if ((Get-RsFolderContent -RsFolder '/' | Where-Object { $_.Name -eq $DataSourceFolderName }) -eq $null)  
            { 
                Write-Host "The folder $DataSourceFolderName doesn't yet exist on the report server. Creating $DataSourceFolderName ..."
                New-RsFolder -Path '/' -Name $DataSourceFolderName -Verbose
            }  
        else
            { Write-Host "An existing folder $DataSourceFolderName was found on the report server." }
        
        #Create the Data Source
        New-RsDataSource -RSfolder "/$DataSourceFolderName" -Name "$DataSourceName" -Description "iService Tenant Reports in $ReportFolderName folder" -Extension 'SQL' -ConnectionString $DSConnectionString -CredentialRetrieval 'Store' -DataSourceCredentials $DSCredential -OverWrite -Verbose
    }
else 
    {
        #Be sure the Data Source the user wants actually exists on the report server before continuing.
        if ((Get-RsFolderContent -Path "/$DataSourceFolderName" | Where-Object {$_.Name -eq $DataSourceName }) -eq $null)
            {
                Throw "The Data Source /$DataSourceFolderName/$DataSourceName wasn't found, so no reports deployed! Use the -DeployDataSource argument if you want to create a new Data Source."
            }

}

#Create the tenant's reports folder on the report server if it doesn't already exist  
if ((Get-RsFolderContent -RsFolder '/' | Where-Object { $_.Name -eq $ReportFolderName }) -eq $null)  
    { 
        Write-Host "The folder $ReportFolderName doesn't yet exist on the report server. Creating $ReportFolderName ..."
        New-RsFolder -Path '/' -Name $ReportFolderName -Verbose
    }  
else
    { Write-Host "An existing folder $ReportFolderName was found on the report server." }

#Deploy the reports 
$ReportsInBuild | ForEach-Object { 
        Write-RsCatalogItem -RsFolder "/$ReportFolderName" -Path $_.FullName -Verbose -Overwrite -WarningAction:SilentlyContinue 
    }

#Deploy image(s)
$ImagesInBuild | ForEach-Object { 
        Write-RsCatalogItem -RsFolder "/$ReportFolderName" -Path $_.FullName -Hidden -Verbose -Overwrite -WarningAction:SilentlyContinue 
    }
#Set data source for the reports
$ReportsOnServer = Get-RsCatalogItems -RsFolder "/$ReportFolderName" | Where-Object TypeName -eq 'Report' 

$ReportsOnServer | ForEach-Object {  
		Set-RsDataSourceReference -Path $_.Path -DataSourceName 'iServiceCRM' -DataSourcePath "/$DataSourceFolderName/$DataSourceName" -Verbose
	}

#Check and warn if there were any reports found on the server, but not in this release.
$ReportsOnServer | ForEach-Object {  
		If ($ReportsInBuild.BaseName -notcontains $_.Name) 
            {
                $ReportsNotInBuild += $_.Name
            }
        
	}

If ($ReportsNotInBuild.Count -ne 0) {
    Write-Host 'NOTICE: The following iService reports were found on your report server, but were not included in the package just deployed. These reports were NOT deleted or removed from your report server.  However, this may indicate that the reports were removed or renamed in this release, and may no longer function as intended. Please refer to release documentation for this version for details on any removed or renamed reports:'
    Write-Host ($ReportsNotInBuild  -join "`n")
    }

